/**
 * Copyright © 2019 František Kučera (Frantovo.cz, GlobalCode.info)
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program. If
 * not, see <http://www.gnu.org/licenses/>.
 */

import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.Channel;
import java.nio.channels.DatagramChannel;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

public class InheritedChannelDemo {

	public static void main(String[] args) throws Exception {

		System.err.println("Java: startujeme");

		Runtime.getRuntime().addShutdownHook(new Thread(() -> {
			System.err.println("Java: ShutdownHook → končíme.");
		}));

		Channel inherited = System.inheritedChannel();

		if (inherited == null) {
			// STDIO je napojeno buď na terminál, běžné soubory nebo roury (<,>,|)
			System.err.println("Java: STDIO není zděděný kanál.");
		} else if (inherited instanceof SocketChannel) {
			// např. při použití xinetd s wait=no
			// socat stdio unix-connect:test.sock
			try (SocketChannel socket = (SocketChannel) inherited) {
				String zpráva = "Java: zděděný kanál je SocketChannel.\n";
				socket.write(ByteBuffer.wrap(zpráva.getBytes()));
				// System.err.print(zpráva);
			}
		} else if (inherited instanceof ServerSocketChannel) {
			// např. při použití xinetd s wait=yes
			// socat stdio unix-connect:test.sock
			ServerSocketChannel server = (ServerSocketChannel) inherited;
			for (int i = 0; i < 3; i++) {  // obsloužíme tři spojení a ukončíme se
				try (SocketChannel socket = server.accept()) { // obsluhujeme v jednu chvíli jen jedno spojení (mohli bychom to dělat vícevláknově)
					String zpráva = "Java: zděděný kanál je ServerSocketChannel, ze kteréhé jsme přijali spojení #" + i + " SocketChannel.\n";
					socket.write(ByteBuffer.wrap(zpráva.getBytes()));
					System.err.print(zpráva);
				}
			}
		} else if (inherited instanceof DatagramChannel) {
			DatagramChannel server = (DatagramChannel) inherited;
			server.configureBlocking(true);
			ByteBuffer datagram = ByteBuffer.allocate(100);
			SocketAddress adresaKlienta = server.receive(datagram);
			System.err.println("Java: zděděný kanál je DatagramChannel, ze kterého jsme přijali datagram: " + new String(datagram.array()));
			// System.err.println("Java: adresa klienta: " + adresaKlienta);
			// server.send(ByteBuffer.wrap(("tvoje adresa: " + adresaKlienta + "\n").getBytes()), adresaKlienta);
			// echo ahoj | socat stdio unix-sendto:test.sock
		} else {
			throw new IllegalStateException("Nepodporovaný typ zděděného kanálu: " + inherited.getClass().getName()); // nemělo by nastat
		}
	}
}
